use actix_web::{web, HttpResponse, Responder};
use diesel::pg::PgConnection;
use diesel::r2d2::{self, ConnectionManager};
use serde::Deserialize;

use super::r#trait::CrudEntity;

type DbPool = r2d2::Pool<ConnectionManager<PgConnection>>;

#[derive(Deserialize)]
struct Pagination {
    limit: Option<i64>,
    offset: Option<i64>,
}

pub async fn get_all<T: CrudEntity>(
    pool: web::Data<DbPool>,
    query: web::Query<Pagination>,
) -> impl Responder {
    let mut conn = match pool.get() {
        Ok(conn) => conn,
        Err(_) => return HttpResponse::InternalServerError().finish(),
    };
    match T::find_all(&mut conn, query.limit, query.offset) {
        Ok(items) => HttpResponse::Ok().json(items),
        Err(_) => HttpResponse::InternalServerError().finish(),
    }
}

pub async fn get_one<T: CrudEntity>(
    pool: web::Data<DbPool>,
    id: web::Path<T::Id>,
) -> impl Responder {
    let mut conn = match pool.get() {
        Ok(conn) => conn,
        Err(_) => return HttpResponse::InternalServerError().finish(),
    };
    match T::find_by_id(&mut conn, id.into_inner()) {
        Ok(item) => HttpResponse::Ok().json(item),
        Err(diesel::result::Error::NotFound) => HttpResponse::NotFound().finish(),
        Err(_) => HttpResponse::InternalServerError().finish(),
    }
}

pub async fn create<T: CrudEntity>(
    pool: web::Data<DbPool>,
    item: web::Json<T::CreateInput>,
) -> impl Responder {
    let mut conn = match pool.get() {
        Ok(conn) => conn,
        Err(_) => return HttpResponse::InternalServerError().finish(),
    };
    match T::insert(&mut conn, item.into_inner()) {
        Ok(created) => HttpResponse::Created().json(created),
        Err(_) => HttpResponse::InternalServerError().finish(),
    }
}

pub async fn update<T: CrudEntity>(
    pool: web::Data<DbPool>,
    id: web::Path<T::Id>,
    item: web::Json<T::UpdateInput>,
) -> impl Responder {
    let mut conn = match pool.get() {
        Ok(conn) => conn,
        Err(_) => return HttpResponse::InternalServerError().finish(),
    };
    match T::update(&mut conn, id.into_inner(), item.into_inner()) {
        Ok(updated) => HttpResponse::Ok().json(updated),
        Err(diesel::result::Error::NotFound) => HttpResponse::NotFound().finish(),
        Err(_) => HttpResponse::InternalServerError().finish(),
    }
}

pub async fn delete<T: CrudEntity>(
    pool: web::Data<DbPool>,
    id: web::Path<T::Id>,
) -> impl Responder {
    let mut conn = match pool.get() {
        Ok(conn) => conn,
        Err(_) => return HttpResponse::InternalServerError().finish(),
    };
    match T::delete(&mut conn, id.into_inner()) {
        Ok(_) => HttpResponse::NoContent().finish(),
        Err(diesel::result::Error::NotFound) => HttpResponse::NotFound().finish(),
        Err(_) => HttpResponse::InternalServerError().finish(),
    }
}